import logging

from django.core.urlresolvers import resolve, reverse
from django.views.decorators.csrf import requires_csrf_token
from django.conf import settings
from django import http

from django_auth_policy.handlers import PasswordChangePolicyHandler
from django_auth_policy.forms import StrictPasswordChangeForm


logger = logging.getLogger(__name__)


class AuthenticationPolicyMiddleware(object):
    """ This middleware enforces the following policy:
    - Change of password when password has expired;
    - Change of password when user has a temporary password;
    - Logout disabled users;

    This is enforced using middleware to prevent users from accessing any page
    handled by Django without the policy being enforced.
    """

    change_password_path = reverse(getattr(
        settings, 'ENFORCED_PASSWORD_CHANGE_VIEW_NAME', 'password_change'))
    login_path = reverse(getattr(settings, 'LOGIN_VIEW_NAME', 'login'))
    logout_path = reverse(getattr(settings, 'LOGOUT_VIEW_NAME', 'logout'))

    password_change_policy_handler = PasswordChangePolicyHandler()

    def process_request(self, request):
        assert hasattr(request, 'user'), (
            'AuthenticationPolicyMiddleware needs a user attribute on '
            'request, add AuthenticationMiddleware before '
            'AuthenticationPolicyMiddleware in MIDDLEWARE_CLASSES')

        if not request.user.is_authenticated():
            return None

        # Log out disabled users
        if not request.user.is_active:
            logger.info('Log out inactive user, user=%s', request.user)
            view_func, args, kwargs = resolve(self.logout_path)
            return view_func(request, *args, **kwargs)

        # Do not do password change for certain URLs
        if request.path in (self.change_password_path, self.login_path,
                            self.logout_path):
            return None

        # Check for 'enforce_password_change' in session set by login view
        if request.session.get('password_change_enforce', False):
            return self.password_change(request)

        return None

    def process_response(self, request, response):
        if not hasattr(request, 'user') or not request.user.is_authenticated():
            return response

        # When password change is enforced, check if this is still required
        # for next request
        if not request.session.get('password_change_enforce', False):
            return response

        self.password_change_policy_handler.update_session(
            request, request.user)

        return response

    def password_change(self, request):
        """ Return 'password_change' view.
        This resolves the view with the name 'password_change'.

        Overwrite this method when needed.
        """
        view_func, args, kwargs = resolve(self.change_password_path)

        if 'password_change_form' in kwargs:
            assert issubclass(kwargs['password_change_form'],
                              StrictPasswordChangeForm), (
                "Use django_auth_policy StrictPasswordChangeForm for password "
                "changes.")

        # Provide extra context to be used in the password_change template
        if 'extra_context' in kwargs:
            kwargs['extra_context']['password_change_enforce'] = \
                request.session.get('password_change_enforce')
            kwargs['extra_context']['password_change_enforce_msg'] = \
                request.session.get('password_change_enforce_msg')

        # Run 'requires_csrf_token' because CSRF middleware might have been
        # skipped over here
        return requires_csrf_token(view_func)(request, *args, **kwargs)


class LoginRequiredMiddleware(object):
    """ Middleware which enforces authentication for all requests.
    """
    login_path = reverse(getattr(settings, 'LOGIN_VIEW_NAME', 'login'))
    logout_path = reverse(getattr(settings, 'LOGOUT_VIEW_NAME', 'logout'))
    public_urls = getattr(settings, 'PUBLIC_URLS', [])
    public_urls.append(login_path)
    public_urls.append(logout_path)

    def process_request(self, request):
        if not hasattr(request, 'user'):
            raise Exception('Install Authentication middleware before '
                            'LoginRequiredMiddleware')

        if request.user.is_authenticated():
            return None

        # Do not require authentication for certain URLs
        if request.path in self.public_urls:
            return None

        # Django should not serve STATIC files in production, but for
        # DEBUG mode this should be no problem (development)
        if (settings.STATIC_URL and
            request.path.startswith(settings.STATIC_URL)):

            if settings.DEBUG:
                return None
            else:
                return http.HttpResponse('Unauthenticated', status=401)

        # When serving MEDIA files through Django we will not display a login
        # form, but instead return HTTP 401, but for DEBUG mode this should be
        # no problem (development)
        if (settings.MEDIA_URL and
            request.path.startswith(settings.MEDIA_URL)):

            if settings.DEBUG:
                return None
            else:
                return http.HttpResponse('Unauthenticated', status=401)

        # Ajax views should not display a login form, we use HTTP 401 to
        # indicate an unauthorized request, like a session timeout
        if request.is_ajax():
            return http.HttpResponse('Unauthenticated', status=401)

        view_func, args, kwargs = resolve(self.login_path)
        return requires_csrf_token(view_func)(request, *args, **kwargs)



import unicodedata
import re

from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from django_auth_policy import BasePolicy


def _normalize_unicode(value):
    try:
        value = unicodedata.normalize('NFKD', unicode(value))
        return value.encode('ascii', 'ignore').strip().lower()
    except UnicodeDecodeError:
        return value


class PasswordStrengthPolicy(BasePolicy):
    """ Password strength policy classes must implement:

    `validate` a method which accept a password and the related user and raises
    a validation error when the password doesn't validate the policy.

    Optionally:

    `policy_text` a property which returns a short text to be displayed in
    password policy explenations

    `policy_caption` a property which returns a short caption to be displayed
    with the password policy.
    """
    show_policy = True

    def validate(self, value, user=None):
        raise NotImplemented()

    @property
    def policy_text(self):
        return None

    @property
    def policy_caption(self):
        return None


class PasswordMinLength(PasswordStrengthPolicy):
    min_length = 10
    text = _('Passwords must be at least {min_length} characters in length.')

    def validate(self, value, user=None):
        if self.min_length is None:
            return

        if len(value) < self.min_length:
            msg = self.text.format(min_length=self.min_length)
            raise ValidationError(msg, code='password_min_length')

    @property
    def policy_text(self):
        return self.text.format(min_length=self.min_length)


class PasswordContains(PasswordStrengthPolicy):
    """ Base class which validates if passwords contain at least a certain
    number of characters from a certain set.
    """
    chars = None
    min_count = 1
    text = None
    plural_text = None

    def validate(self, value, user=None):
        pw_set = set(value)
        if len(pw_set.intersection(self.chars)) < self.min_count:
            raise ValidationError(self.text, 'password_complexity')

    @property
    def policy_text(self):
        if self.min_count > 1:
            return self.plural_text.format(min_count=self.min_count)
        else:
            return self.text.format(min_count=self.min_count)

    @property
    def policy_caption(self):
        return self.chars


class PasswordContainsUpperCase(PasswordContains):
    chars = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'
    text = _('Passwords must have at least one uppercase character.')
    plural_text = _('Passwords must have at least {min_count} '
                    'uppercase characters.')


class PasswordContainsLowerCase(PasswordContains):
    chars = 'abcdefghijklmnopqrstuvwxyz'
    text = _('Passwords must have at least one lowecase character.')
    plural_text = _('Passwords must have at least {min_count} '
                    'lowercase characters.')


class PasswordContainsNumbers(PasswordContains):
    chars = '0123456789'
    text = _('Passwords must have at least one number.')
    plural_text = _('Passwords must have at least {min_count} '
                    'numbers.')


class PasswordContainsSymbols(PasswordContains):
    chars = '!@#$%^&*()_+-={}[]:;"\'|\\,.<>?/~` '
    text = _('Passwords must have at least one special character (punctuation).')
    plural_text = _('Passwords must have at least {min_count} special '
                    'characters (punctuation).')


class PasswordContainsAlphabetics(PasswordContains):
    chars = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
    text = _('Passwords must have at least one alphabetic character.')
    plural_text = _('Passwords must have at least {min_count} '
                    'alphabetic characters.')


class PasswordUserAttrs(PasswordStrengthPolicy):
    """ Validate if password doesn't contain values from a list of user
    attributes. Every attribute will be normalized into ascii and split
    on non alphanumerics.

    Use this in the clean method of password forms

    `value`: password
    `user`: user object with attributes

    Example, which would raise a ValidationError:

        user.first_name = 'John'
        password_user_attrs('johns_password', user)
    """
    user_attrs = ('email', 'first_name', 'last_name', 'username')
    text = _('Passwords are not allowed to contain (pieces of) your name '
             'or email.')

    _non_alphanum = re.compile(r'[^a-z0-9]')

    def validate(self, value, user=None):
        if user is None:
            return

        simple_pass = _normalize_unicode(value)
        for attr in self.user_attrs:
            v = getattr(user, attr, None)
            if not attr or len(attr) < 4:
                continue

            v = _normalize_unicode(v)

            for piece in self._non_alphanum.split(v):
                if len(piece) < 4:
                    continue

                if piece in simple_pass:
                    raise ValidationError(self.text, 'password_user_attrs')

    @property
    def policy_text(self):
        return self.text


class PasswordDisallowedTerms(PasswordStrengthPolicy):
    """ Disallow a (short) list of terms in passwords
    Ideal for too obvious terms like the name of the site or company
    """
    terms = None
    text = _('Passwords are not allowed to contain the following term(s): '
             '{terms}.')
    show_policy = False

    def __init__(self, **kwargs):
        terms = kwargs.pop('terms')
        self.terms = [_normalize_unicode(term) for term in terms]

        super(PasswordDisallowedTerms, self).__init__(**kwargs)

    def validate(self, value, user=None):
        simple_pass = _normalize_unicode(value)
        found = []
        for term in self.terms:
            if term in simple_pass:
                found.append(term)

        if found:
            msg = self.text.format(terms=u', '.join(found))
            raise ValidationError(msg, 'password_disallowed_terms')

    @property
    def policy_text(self):
        return self.text.format(terms=u', '.join(self.terms))



from django.core.management.base import BaseCommand, CommandError
from django.core.exceptions import ValidationError
from django.conf import settings

from django_auth_policy.handlers import PasswordStrengthPolicyHandler


class Command(BaseCommand):
    help = ("Filters a list of password by removing all passwords that do not "
            "comply with the configured password strength policies."
            "Provide file paths as arguments.")

    def handle(self, *args, **options):
        if not args:
            print ("Please provide one or more file paths for files with "
                   "password lists.")
            return

        pw_handler = PasswordStrengthPolicyHandler()

        for arg in args:
            fh = open(arg, 'r')
            for pw in fh:
                try:
                    pw = pw.decode('utf8')
                except UnicodeDecodeError:
                    continue

                pw = pw.strip()

                try:
                    pw_handler.validate(pw)
                except ValidationError:
                    continue

                print pw.encode('utf8')

            fh.close()



import datetime

from django.utils.translation import ugettext_lazy as _
from django.utils import timezone
from django.core.exceptions import ValidationError

from django_auth_policy import BasePolicy


class PasswordChangePolicy(BasePolicy):
    """ Policies that require a user to change its password
    `change_password` which raises a ValidationError when a user is
    enforced to change its password. Returns None otherwise
    """
    def validate(self, last_pw_change):
        raise NotImplemented()


class PasswordChangeExpired(PasswordChangePolicy):
    """ Enforces expired password to be changed.
    """
    # Password expiration period in days
    max_age = 90
    policy_text = _('One is required to change passwords every {age} days.')
    text = _('Your password has expired and must be changed.')
    allow_empty_password_history = False

    def validate(self, last_pw_change):
        if last_pw_change is None:
            # Enforce a password change when user has no PasswordChange
            # This will happen when introducing Django auth policy or for
            # new users
            if not self.allow_empty_password_history:
                raise ValidationError(self.text, code='password-expired')
            else:
                return

        expire_at = (last_pw_change.timestamp +
                     datetime.timedelta(days=self.max_age))

        if timezone.now() > expire_at:
            raise ValidationError(self.text, code='password-expired')


class PasswordChangeTemporary(PasswordChangePolicy):
    """ Enforces temporary passwords to be changed
    """
    text = _('Your must change your temporary password into a '
             'personal password.')

    def validate(self, last_pw_change):
        if last_pw_change is not None and last_pw_change.is_temporary:
            raise ValidationError(self.text, code='password-temporary')



import datetime
import logging
import re

from django.utils import timezone
from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext_lazy as _

from django_auth_policy import signals
from django_auth_policy.models import LoginAttempt
from django_auth_policy import BasePolicy


logger = logging.getLogger(__name__)


class AuthenticationPolicy(BasePolicy):
    """ Checks run when authenticating.
    Policies can define:

    `pre_auth_check` for checks that should be run before attempting to
    authenticate provided credentials.

    `post_auth_check` for checks that should be run after attempting to
    authenticate provided credentials.

    Both `pre_auth_check` and `post_auth_check` raise a ValidationError
    when authentication fails to comply with the policy

    `auth_success` is run when the attempt was successful and should not
    raise a ValidationError
    """
    def pre_auth_check(self, loginattempt, password):
        pass

    def post_auth_check(self, loginattempt):
        pass

    def auth_success(self, loginattempt):
        pass


class AuthenticationBasicChecks(AuthenticationPolicy):
    text = _("Please enter a correct username and password. "
             "Note that both fields may be case-sensitive.")

    def pre_auth_check(self, loginattempt, password):
        if not loginattempt.username:
            logger.info(u'Authentication failure, address=%s, '
                        'no username supplied.',
                        loginattempt.source_address)
            raise ValidationError(self.text, code='invalid_login')

        if not password:
            logger.info(u'Authentication failure, username=%s, '
                        'address=%s, no password supplied.',
                        loginattempt.username,
                        loginattempt.source_address)
            raise ValidationError(self.text, code='invalid_login')

    def post_auth_check(self, loginattempt):
        if loginattempt.user is None:
            logger.info(u'Authentication failure, username=%s, '
                        'address=%s, invalid authentication.',
                        loginattempt.username, loginattempt.source_address)
            raise ValidationError(self.text, code='invalid_login')

        if not loginattempt.user.is_active:
            logger.info(u'Authentication failure, username=%s, '
                        'address=%s, user inactive.',
                        loginattempt.username, loginattempt.source_address)
            raise ValidationError(self.text, code='inactive')


class AuthenticationDisableExpiredUsers(AuthenticationPolicy):
    """ Disable ALL users that have been expired

    Users must have an `is_active` and a `last_login` field

    Reactivate user by setting is_active to True and last_login to
    now.
    """
    # Days after which users without a successful login expire, make sure
    # user sessions are short enough to enforce frequent re-logins
    inactive_period = 90

    def pre_auth_check(self, loginattempt, password):
        expire_at = timezone.now() - datetime.timedelta(
            days=self.inactive_period)

        expired = get_user_model().objects.filter(is_active=True,
                                                  last_login__lt=expire_at)

        for user in expired:
            logger.info('User %s disabled because last login was at %s',
                        unicode(user), user.last_login)
            # Send signal to be used to alert admins
            signals.user_expired.send(sender=user, user=user)

        expired.update(is_active=False)


def _format_lockduration(seconds):
    duration = datetime.timedelta(seconds=seconds)
    if duration.days > 1:
        return _('{days} days').format(days=duration.days)
    elif duration.days == 1:
        return _('a day')
    elif duration.seconds >= 120:
        return _('{mins} minutes').format(mins=duration.seconds // 60)
    elif duration.seconds >= 60:
        return _('a minute')
    else:
        return _('{secs} seconds').format(secs=duration.seconds)


class AuthenticationLockedUsername(AuthenticationPolicy):
    """ Lockout usernames with too many failed login attempts within a certain
    period.
    """
    # Number of failed login attempts
    max_failed = 3
    # Period in seconds used to count number of failed login attempts,
    # None = indefinite
    period = None
    # Lockout duration in seconds
    lockout_duration = 60 * 10
    # Validation error
    text = _('Too many failed login attempts. Your account has been locked '
             'for {duration}.')

    def pre_auth_check(self, loginattempt, password):
        try:
            prev_login = LoginAttempt.objects.filter(
                username=loginattempt.username).exclude(
                pk=loginattempt.pk).order_by('-id')[0]
        except IndexError:
            # No login attempts for this username and thus no lockout
            return

        # If previous login did not count towards a lockout one is certainly
        # not locked out
        if not prev_login.lockout:
            return

        # If previous login was before lockout duration one is not
        # locked out (anymore)
        lock_from = (timezone.now() -
                     datetime.timedelta(seconds=self.lockout_duration))
        if prev_login.timestamp < lock_from:
            return

        # Count number of locking login attempts
        user_lockout = LoginAttempt.objects.filter(
            username=loginattempt.username, successful=False,
            lockout=True).exclude(pk=loginattempt.pk)

        if self.period is not None:
            lockout_count_from = timezone.now() - datetime.timedelta(
                seconds=self.period)
            user_lockout = user_lockout.filter(
                timestamp__gt=lockout_count_from)

        if user_lockout.count() >= self.max_failed:
            logger.info(u'Authentication failure, username=%s, address=%s, '
                        'username locked', loginattempt.username,
                        loginattempt.source_address)
            raise ValidationError(self.validation_msg,
                                  code='username_locked_out')

    def auth_success(self, loginattempt):
        # Reset lockout counts for username
        LoginAttempt.objects.filter(username=loginattempt.username,
                                    lockout=True).update(lockout=False)

    @property
    def validation_msg(self):
        dur = _format_lockduration(self.lockout_duration)
        return self.text.format(duration=dur)


class AuthenticationLockedRemoteAddress(AuthenticationPolicy):
    """ Lockout IP addresses with too many failed login attempts within a
    certain period.
    """
    # Number of failed login attempts
    max_failed = 3
    # Period in seconds used to count number of failed login attempts
    period = None
    # Lockout duration in seconds
    lockout_duration = 60 * 10
    # Validation error
    text = _('Too many failed login attempts. Your account has been locked '
             'for {duration}.')

    def pre_auth_check(self, loginattempt, password):
        try:
            prev_login = LoginAttempt.objects.filter(
                source_address=loginattempt.source_address).exclude(
                pk=loginattempt.pk).order_by('-id')[0]
        except IndexError:
            # No login attempts for this username and thus no lockout
            return

        # If previous login did not count towards a lockout one is certainly
        # not locked out
        if not prev_login.lockout:
            return

        # If previous login was before lockout duration one is not
        # locked out (anymore)
        lock_from = (timezone.now() -
                     datetime.timedelta(seconds=self.lockout_duration))
        if prev_login.timestamp < lock_from:
            return

        # Count number of locking login attempts
        user_lockout = LoginAttempt.objects.filter(
            source_address=loginattempt.source_address, successful=False,
            lockout=True).exclude(pk=loginattempt.pk)

        if self.period is not None:
            lockout_count_from = timezone.now() - datetime.timedelta(
                seconds=self.period)
            user_lockout = user_lockout.filter(
                timestamp__gt=lockout_count_from)

        if user_lockout.count() >= self.max_failed:
            logger.info(u'Authentication failure, username=%s, address=%s, '
                        'address locked',
                        loginattempt.username,
                        loginattempt.source_address)
            raise ValidationError(self.validation_msg,
                                  code='address_locked_out')

    def auth_success(self, loginattempt):
        # Reset lockout counts for password
        LoginAttempt.objects.filter(source_address=loginattempt.source_address,
                                    lockout=True).update(lockout=False)

    @property
    def validation_msg(self):
        dur = _format_lockduration(self.lockout_duration)
        return self.text.format(duration=dur)


class AuthenticationUsernameWhitelist(AuthenticationPolicy):
    """ Only allow usernames that match regular expressions
    Useful to restrict login with email addresses with a certain domainname
    """
    # Regexes
    whitelist = []
    _whitelist_regex = []
    text = _("Please enter a correct username and password. "
             "Note that both fields may be case-sensitive.")

    def pre_auth_check(self, loginattempt, password):
        if not self._whitelist_regex:
            for pattern in self.whitelist:
                self._whitelist_regex.append(re.compile(pattern))

        for regex in self._whitelist_regex:
            if regex.search(loginattempt.username):
                logger.debug('Username matched whitelisted pattern %s',
                             regex.pattern)
                return

        logger.info(u'Authentication failure, username %s did not match '
                    'whitelisted pattern(s)', loginattempt.username)
        raise ValidationError(self.text, code='invalid_login')



class BasePolicy(object):
    def __init__(self, **kwargs):
        """ Policy configuration happens at init
        """
        for key, value in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)



from django.core.management.base import BaseCommand, CommandError
from django.conf import settings

from django_auth_policy.models import LoginAttempt


class Command(BaseCommand):
    help = "Remove all locks on usernames and IP addresses"

    def handle(self, *args, **options):
        c = LoginAttempt.objects.filter(lockout=True).update(lockout=False)
        print u'Unlocked {0} login attempts.'.format(c)



#!/usr/bin/env python

from setuptools import setup


install_requires = ['Django>=1.5']
try:
    from collections import OrderedDict
except ImportError:
    install_requires.append('ordereddict>=1.1')


setup(
    name='django-auth-policy',
    version='0.9.3',
    zip_safe=False,
    description='Enforces a couple of common authentication policies for the '
                'Django web framework.',
    author='Fox-IT B.V.',
    author_email='fox@fox-it.com',
    maintainer='Rudolph Froger',
    maintainer_email='rudolphfroger@estrate.nl',
    url='https://github.com/rudolphfroger/django-auth-policy',
    license='BSD',
    packages=['django_auth_policy'],
    package_data={'django_auth_policy': ['locale/*/LC_MESSAGES/*.mo',
                                         'locale/*/LC_MESSAGES/*.po']},
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
        'Topic :: Security',
        'Topic :: Internet :: WWW/HTTP :: Session',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
)



from django import dispatch

# user_expired is send as soon as a user expires
# It can be used to inform the user or an admin about the expiry
user_expired = dispatch.Signal(providing_args=["user"])

# temporary_password_set is send when a user gets a temporary password
# It can be used to inform the user about the temporary password
temporary_password_set = dispatch.Signal(providing_args=["user", "request",
                                                         "password"])



import logging

from django import http
from django.utils.translation import ugettext as _
from django.core.urlresolvers import reverse
from django.contrib import admin
from django.contrib.auth.views import password_change
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.conf import settings

from django_auth_policy.models import PasswordChange, LoginAttempt, UserChange
from django_auth_policy.forms import (StrictAuthenticationForm,
                                      StrictPasswordChangeForm)


logger = logging.getLogger(__name__)


class LoginAttemptAdmin(admin.ModelAdmin):
    readonly_fields = ('username', 'source_address', 'hostname', 'successful',
                       'user', 'timestamp', 'lockout')
    list_display = ('username', 'source_address', 'successful', 'timestamp')
    list_filter = ('successful',)
    search_fields = ('username',)
    date_hierarchy = 'timestamp'
    actions = ('unlock',)

    def get_actions(self, request):
        # Disable deletion of login attempts
        actions = super(LoginAttemptAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']

        if not request.user.has_perm('django_auth_policy.unlock'):
            del actions['unlock']

        return actions

    def unlock(self, request, queryset):
        if not request.user.has_perm('django_auth_policy.unlock'):
            self.message_user(request, _('You don\'t have unlock permissions'))
            return
        count = LoginAttempt.objects.unlock_queryset(queryset)
        self.message_user(request, _('Unlocked %s failed attempts') % count)
    unlock.short_description = _('Unlock')

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def save_model(self, request, obj, form, change):
        # Do not actually save anything to prevent changes
        logger.info('Prevented change in LoginAttempt item by user %s',
                    request.user)


class PasswordChangeAdmin(admin.ModelAdmin):
    readonly_fields = ('user', 'timestamp', 'successful', 'is_temporary')
    list_display = ('user', 'successful', 'is_temporary', 'timestamp')
    list_filter = ('successful', 'is_temporary')
    date_hierarchy = 'timestamp'

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def save_model(self, request, obj, form, change):
        # Do not actually save anything to prevent changes
        logger.info('Prevented change in PasswordChange item by user %s',
                    request.user)

    def get_actions(self, request):
        # Disable deletion of user changes action
        actions = super(PasswordChangeAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions


class UserChangeAdmin(admin.ModelAdmin):
    readonly_fields = ('user', 'timestamp', 'by_user')
    list_display = ('user', 'by_user', 'timestamp')
    date_hierarchy = 'timestamp'

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        return False

    def save_model(self, request, obj, form, change):
        # Do not actually save anything to prevent changes
        logger.info('Prevented change in UserChange item by user %s',
                    request.user)

    def get_actions(self, request):
        # Disable deletion of user changes action
        actions = super(UserChangeAdmin, self).get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions


def admin_login(request, extra_context=None):
    """ Redirects to default login view which enforces the authentication
    policy
    """
    q = REDIRECT_FIELD_NAME + '=' + request.get_full_path()
    return http.HttpResponseRedirect(reverse('login') + '?' + q)


def admin_password_change(request):
    """
    Handles the "change password" task -- both form display and validation.
    """
    to_url = reverse('admin:password_change_done', current_app=admin.site.name)
    defaults = {
        'current_app': admin.site.name,
        'post_change_redirect': to_url,
        'password_change_form': StrictPasswordChangeForm
    }
    if admin.site.password_change_template is not None:
        defaults['template_name'] = admin.site.password_change_template
    return password_change(request, **defaults)


admin.site.register(LoginAttempt, LoginAttemptAdmin)
admin.site.register(PasswordChange, PasswordChangeAdmin)
admin.site.register(UserChange, UserChangeAdmin)

admin.site.login = admin_login
admin.site.login_form = StrictAuthenticationForm
admin.site.password_change = admin_password_change

# django_auth_policy replaces the default Django auth UserAdmin to enforce
# authentication policies on the admin interface. Set this to False when
# django_auth_policy shouldn't replace UserAdmin.
replace_auth_user_admin = getattr(settings, 'REPLACE_AUTH_USER_ADMIN', True)

# UserAdmin is never replaced when a custom Auth model is in use
if replace_auth_user_admin and settings.AUTH_USER_MODEL == 'auth.User':
    # Import this here to avoid unwanted registering of django auth admin
    from django.contrib.auth import get_user_model
    from django_auth_policy.user_admin import StrictUserAdmin
    user_model = get_user_model()
    try:
        admin.site.unregister(user_model)
    except admin.sites.NotRegistered:
        logger.warning('Could not unregister django.contrib.auth User model '
                       'from admin site. Make sure your custom user admin '
                       'enforces all authentication policies and set '
                       'REPLACE_AUTH_USER_ADMIN to False.')
    else:
        admin.site.register(user_model, StrictUserAdmin)



import django
from django.core.management.base import BaseCommand, CommandError
from django.core.urlresolvers import reverse, resolve
from django.conf import settings
from django.contrib import admin
from django.contrib.auth import get_user_model

from django_auth_policy.views import login
from django_auth_policy.user_admin import StrictUserAdmin
from django_auth_policy.admin import (LoginAttemptAdmin, admin_login,
                                      admin_password_change)
from django_auth_policy.forms import (StrictAuthenticationForm,
                                      StrictPasswordChangeForm)

user_model = get_user_model()


class Command(BaseCommand):
    help = "Checks if all settings for django_auth_policy are in place."

    def handle(self, *args, **options):
        self.check_middleware()
        self.check_settings()
        self.views()
        self.admin()

    def check_middleware(self):
        middle = 'django_auth_policy.middleware.AuthenticationPolicyMiddleware'
        d_middle = 'django.contrib.auth.middleware.AuthenticationMiddleware'
        m_classes = list(settings.MIDDLEWARE_CLASSES)
        if not middle in m_classes:
            self.stderr.write('WARNING: AuthenticationPolicyMiddleware is '
                              'missing in MIDDLEWARE_CLASSES')

        elif not d_middle in m_classes:
            self.stderr.write('WARNING: Djangos AuthenticationMiddleware is '
                              'missing in MIDDLEWARE_CLASSES')

        elif m_classes.index(d_middle) > m_classes.index(middle):
            self.stderr.write('WARNING: Djangos AuthenticationMiddleware must '
                              'come before AuthenticationPolicyMiddleware in '
                              'MIDDLEWARE_CLASSES')


        if not 'django.middleware.csrf.CsrfViewMiddleware' in m_classes:
            self.stderr.write('WARNING: CsrfViewMiddleware is missing in '
                              'MIDDLEWARE_CLASSES')

        clickjack = 'django.middleware.clickjacking.XFrameOptionsMiddleware'
        if not clickjack in m_classes:
            self.stderr.write('WARNING: clickjacking protection using '
                              'XFrameOptionsMiddleware missing in '
                              'MIDDLEWARE_CLASSES')

    def check_settings(self):
        if not settings.X_FRAME_OPTIONS in ('DENY', 'SAMEORIGIN'):
            self.stderr.write('WARNING: X_FRAME_OPTIONS set to %s' %
                              settings.X_FRAME_OPTIONS)

        if not settings.SESSION_COOKIE_SECURE:
            self.stderr.write('WARNING: SESSION_COOKIE_SECURE is set to False')

        if not settings.SESSION_COOKIE_HTTPONLY:
            self.stderr.write('WARNING: SESSION_COOKIE_HTTPONLY is set to False')

        if settings.SESSION_COOKIE_AGE > 60 * 60 * 24:
            self.stderr.write('WARNING: SESSION_COOKIE_AGE is set to more '
                              'than 24 hours')
        else:
            self.stdout.write('INFO: SESSION_COOKIE_AGE is set to %d seconds' %
                              settings.SESSION_COOKIE_AGE)

        if not settings.CSRF_COOKIE_SECURE:
            self.stderr.write('WARNING: CSRF_COOKIE_SECURE is set to False')

        if (django.VERSION[0] >= 1 and django.VERSION[1] >= 6 and
            not settings.CSRF_COOKIE_HTTPONLY):

            self.stderr.write('WARNING: CSRF_COOKIE_HTTPONLY is set to False')

        if settings.DEBUG:
            self.stderr.write('WARNING: DEBUG is set to True')

        if settings.TEMPLATE_DEBUG:
            self.stderr.write('WARNING: TEMPLATE_DEBUG is set to True')


    def views(self):
        # Check login view
        url = reverse('login')
        if url != settings.LOGIN_URL:
            self.stderr.write('WARNING: login URL not equal to '
                              'settings.LOGIN_URL')

        func, args, kwargs = resolve(url)
        if func != login:
            self.stderr.write('WARNING: login view isn\'t the '
                              'django_auth_policy.views.login view')

        if 'authentication_form' in kwargs:
            if not issubclass(kwargs['authentication_form'],
                              StrictAuthenticationForm):
                self.stderr.write('WARNING: login view doesn\'t use '
                                  'the StrictAuthenticationForm')
        else:
            self.stderr.write('WARNING: could not check if login view uses '
                              'the StrictAuthenticationForm')

        # Check password change view
        url = reverse(getattr(settings, 'ENFORCED_PASSWORD_CHANGE_VIEW_NAME',
                              'password_change'))
        func, args, kwargs = resolve(url)
        if 'password_change_form' in kwargs:
            if not issubclass(kwargs['password_change_form'],
                              StrictPasswordChangeForm):
                self.stderr.write('WARNING: password_change view doesn\'t use '
                                  'the StrictPasswordChangeForm')
        else:
            self.stderr.write('WARNING: could not check if password_change '
                              'view uses the StrictAuthenticationForm')

    def admin(self):
        if not 'django.contrib.admin' in settings.INSTALLED_APPS:
            return

        # Check login view
        if admin.site.login != admin_login:
            self.stderr.write('WARNING: admin doesn\'t use the '
                              'django_auth_policy admin_login view')

        if not issubclass(admin.site.login_form, StrictAuthenticationForm):
            self.stderr.write('WARNING: admin doesn\'t use the '
                              'django_auth_policy StrictAuthenticationForm')

        # Check password change
        if not admin.site.password_change == admin_password_change:
            self.stderr.write('WARNING: admin doesn\'t use the '
                              'django_auth_policy admin_password_change view')

        # Check UserAdmin
        if not getattr(settings, 'REPLACE_AUTH_USER_ADMIN', True):
            # Replacing disabled
            pass
        elif not user_model in admin.site._registry:
            self.stderr.write('WARNING: Could not check if user admin '
                              'doesn\'t use django_auth_policy\'s '
                              'StrictUserAdmin')
        elif not isinstance(admin.site._registry[user_model], StrictUserAdmin):
            self.stderr.write('WARNING: User admin doesn\'t use '
                              'django_auth_policy\'s StrictUserAdmin')
